#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>

#include "eit.h"

char		*subtitle_cnv_str[] = {
	NULL
};


int parseEIThead(unsigned char *data, EIThead *h) {
	int boff = 0;

	memset(h, 0, sizeof(EIThead));

	h->table_id = getBit(data, &boff, 8);
	h->section_syntax_indicator = getBit(data, &boff, 1);
	h->reserved_future_use = getBit(data, &boff, 1);
	h->reserved1 = getBit(data, &boff, 2);
	h->section_length =getBit(data, &boff,12);
	h->service_id = getBit(data, &boff, 16);
	h->reserved2 = getBit(data, &boff, 2);
	h->version_number = getBit(data, &boff, 5);
	h->current_next_indicator = getBit(data, &boff, 1);
	h->section_number = getBit(data, &boff, 8);
	h->last_section_number = getBit(data, &boff, 8);
	h->transport_stream_id = getBit(data, &boff, 16);
	h->original_network_id = getBit(data, &boff, 16);
	h->segment_last_section_number = getBit(data, &boff, 8);
	h->last_table_id = getBit(data, &boff, 8);
  
	return 14;
}

int parseEITbody(unsigned char *data, EITbody *b)
{
	int				tnum;
	int				boff = 0;
	unsigned char	start_time[5];
	unsigned char	duration[3];

	memset(b, 0, sizeof(EITbody));

	b->event_id = getBit(data, &boff, 16);

	memcpy(start_time, data + boff / 8, 5);
	boff += 8 * 5;
	memcpy(duration, data + boff / 8, 3);
	boff += 8 * 3;
	b->running_status = getBit(data, &boff, 3);
	b->free_CA_mode = getBit(data, &boff, 1);
	b->descriptors_loop_length = getBit(data, &boff, 12);

	/* 日付変換 */
	if( start_time[0]==0xFF && start_time[1]==0xFF && start_time[2]==0xFF && start_time[3]==0xFF && start_time[4]==0xFF ){
		b->event_status |= START_TIME_UNCERTAINTY;
	}else{
		tnum = (unsigned int)start_time[0] << 8 | (unsigned int)start_time[1];

		b->yy = (tnum - 15078.2) / 365.25;
		b->mm = ((tnum - 14956.1) - (int)(b->yy * 365.25)) / 30.6001;
		b->dd = (tnum - 14956) - (int)(b->yy * 365.25) - (int)(b->mm * 30.6001);

		if(b->mm == 14 || b->mm == 15) {
			b->yy += 1;
			b->mm = b->mm - 1 - (1 * 12);
		} else {
			b->mm = b->mm - 1;
		}

		b->hh = (int)(start_time[2]>>4) * 10 + (int)(start_time[2]&0x0FU);
		b->hm = (int)(start_time[3]>>4) * 10 + (int)(start_time[3]&0x0FU);
		b->ss = (int)(start_time[4]>>4) * 10 + (int)(start_time[4]&0x0FU);
	}

	if((duration[0] == 0xFF) && (duration[1] == 0xFF) && (duration[2] == 0xFF)){
		b->event_status |= DURATION_UNCERTAINTY;
	}else{
		b->duration =	( (int)(duration[0]>>4) * 10 + (int)(duration[0]&0x0FU) ) * 3600 +
						( (int)(duration[1]>>4) * 10 + (int)(duration[1]&0x0FU) ) * 60 +
						( (int)(duration[2]>>4) * 10 + (int)(duration[2]&0x0FU) );
	}
	return 12;
}

//短形式イベント
int parseSEVTdesc(unsigned char *data, SEVTdesc *desc) {
	int boff = 0;
  
	memset(desc, 0, sizeof(SEVTdesc));

	desc->descriptor_tag = getBit(data, &boff, 8);
	if((desc->descriptor_tag & 0xFF) != 0x4D) {
		return -1;
	}
	desc->descriptor_length = getBit(data, &boff, 8);
	memcpy(desc->ISO_639_language_code, data + boff / 8, 3);
	/* desc->ISO_639_language_code = getBit(data, &boff, 24); */
	boff += 24;
	desc->event_name_length = getBit(data, &boff, 8);
	getStr(desc->event_name, data, &boff, desc->event_name_length);
	desc->text_length = getBit(data, &boff, 8);
	getStr(desc->text, data, &boff, desc->text_length);

	return desc->descriptor_length + 2;
}

//コンテント記述子
int parseContentDesc(unsigned char *data, ContentDesc *desc) {
	int boff = 0;
  
	memset(desc, 0, sizeof(ContentDesc));

	desc->descriptor_tag = getBit(data, &boff, 8);
	if((desc->descriptor_tag & 0xFF) != 0x54) {
		return -1;
	}
	desc->descriptor_length = getBit(data, &boff, 8);
	memcpy(desc->content, data+(boff/8), desc->descriptor_length);
	//getStr(desc->content, data, &boff, desc->descriptor_length);
	return desc->descriptor_length + 2;
}

//シリーズ記述子
int parseSeriesDesc(unsigned char *data, SeriesDesc *desc) {
	int boff = 0;
  
	memset(desc, 0, sizeof(SeriesDesc));

	desc->descriptor_tag = getBit(data, &boff, 8);
	if((desc->descriptor_tag & 0xFF) != 0xD5) {
		return -1;
	}
	desc->descriptor_length = getBit(data, &boff, 8);
	desc->series_id = getBit(data, &boff, 16);
	desc->repeat_label = getBit(data, &boff, 4);
	desc->program_pattern = getBit(data, &boff, 3);
	desc->expire_date_valid_flag = getBit(data, &boff, 1);

	desc->expire_date = getBit(data, &boff, 16);
	//memcpy(desc->expire_date, data + boff / 8, 2);
	//boff += 16;

	desc->episode_number = getBit(data, &boff, 12);
	desc->last_episode_number = getBit(data, &boff, 12);

	getStr(desc->series_name_char, data, &boff, desc->descriptor_length - 8);
	return desc->descriptor_length + 2;
}

//拡張形式イベント
int parseEEVTDhead(unsigned char *data, EEVTDhead *desc) {
	int boff = 0;
  
	memset(desc, 0, sizeof(EEVTDhead));

	desc->descriptor_tag = getBit(data, &boff, 8);
	if((desc->descriptor_tag & 0xFF) != 0x4E) {
		return -1;
	}
	desc->descriptor_length = getBit(data, &boff, 8);
	desc->descriptor_number = getBit(data, &boff, 4);
	desc->last_descriptor_number = getBit(data, &boff, 4);
	memcpy(desc->ISO_639_language_code, data + boff / 8, 3);
	/* desc->ISO_639_language_code = getBit(data, &boff, 24); */
	boff += 24;

	desc->length_of_items = getBit(data, &boff, 8);

	return 7;
}

int parseEEVTDitem(unsigned char *data, EEVTDitem *desc) {
	int boff = 0;
  
	memset(desc, 0, sizeof(EEVTDitem));

	desc->item_description_length = getBit(data, &boff, 8);
	getStr(desc->item_description, data, &boff, desc->item_description_length);

	desc->item_length = getBit(data, &boff, 8);
	memcpy(desc->item, data + (boff / 8), desc->item_length);
	/* getStr(desc->item, data, &boff, desc->item_length); */

	return desc->item_description_length + desc->item_length + 2;
}

int parseEEVTDtail(unsigned char *data, EEVTDtail *desc) {
	int boff = 0;
  
	memset(desc, 0, sizeof(EEVTDtail));

	desc->text_length = getBit(data, &boff, 8);
	getStr(desc->text, data, &boff, desc->text_length);

	return desc->text_length + 1;
}

int checkEEVTDitem(EEVTDitem *save, EEVTDitem *new, int descriptor_number) {

	EEVTDitem swap;
	int boff = 0;

	if(new == NULL) {
		if(save->item_length != 0) {
			swap = *save;
			getStr(save->item, (unsigned char*)swap.item, &boff, swap.item_length);
			return 1;
		} else {
			return 0;
		}
	}

	if(new->item_description_length == 0) {
		/* 続き 保存 */
		memcpy(save->item + save->item_length, new->item, new->item_length);
		save->item_length += new->item_length;
		return 0;
	} else {
		/* ブレーク。saveを印刷対象にする。saveをクリア? */
		if(save->item_length != 0) {
			/* 退避済みがあり */
			swap = *save;
			getStr(save->item, (unsigned char*)swap.item, &boff, swap.item_length);
			swap = *new;
			*new = *save;
			*save = swap;
			save->descriptor_number = descriptor_number;
		} else {
			*save = *new;
			save->descriptor_number = descriptor_number;
			return 0;
		}
	}

	return 1;
}
EIT_CONTROL	*searcheit(EIT_CONTROL *top, int servid, int eventid)
{
	EIT_CONTROL	*cur ;
	cur = top ;

	while(cur != NULL){
		if((cur->event_id == eventid) && (cur->servid == servid)){
			return cur ;
		}

		cur = cur->next ;
	}
	return NULL ;
}
char	*strstr_eucjp(const char *str, const char *search)
{
	char *pos ;
	pos = (char *)str ;

	while (*pos != '\0') {
		if (*pos == *search) {
			if (strncmp(pos, search, strlen(search)) == 0) {
				return pos ;
			}
		}
		if (*(unsigned char *)pos == 0x8Fu) {
			pos += 3 ;
		} else if (*(unsigned char *)pos >= 0x80u) {
			pos += 2 ;
		} else {
			pos += 1 ;
		}
	}

	return NULL ;
}
void	conv_title_subtitle(EIT_CONTROL *eitptr)
{
	int		lp = 0 ;
//	size_t	addsize ;
	char	*ptr ;
	char	*ptr2 ;
	char	*newsubtitle ;

	for(lp = 0 ; subtitle_cnv_str[lp] != NULL ; lp++){
		ptr = strstr(eitptr->title, subtitle_cnv_str[lp]);
		if(ptr == NULL){
			continue ;
		}
		// タイトルがなくならないように
		if(ptr == eitptr->title){
			continue ;
		}
		ptr2 = ptr ;
		for( ; (unsigned char)*ptr2 == 0x20u ; ptr2++ );
		for( ; (unsigned char)*ptr2 == 0xA1u && (unsigned char)*(ptr2+1) == 0xA1u ; ptr2 += 2);
		for( ; (unsigned char)*ptr2 == 0x20u ; ptr2++ );
		newsubtitle = calloc(1, ((strlen(ptr2) + 2) + (strlen(eitptr->subtitle) + 1)));
		memcpy(newsubtitle, ptr2, strlen(ptr2));
//		*(newsubtitle+strlen(ptr)) = ' ';
		strcat(newsubtitle, "▽");
		*ptr = '\0';
		strcat(newsubtitle, eitptr->subtitle);
		free(eitptr->subtitle);
		eitptr->subtitle = newsubtitle ;
		return ;
	}
}


EIT_CONTROL	*Start_eit = NULL;
int			Start_eid = 0;
int			Start_sid = 0;


void	enqueue(EIT_CONTROL *top, EIT_CONTROL *eitptr)
{
	EIT_CONTROL	*cur ;
	EIT_CONTROL	*tmp;
	int		rc ;
	int		duration = 0;
	time_t	end_tm;

	if(top->next == NULL){
		if( eitptr->table_id != 0x4EU ){
			Start_eit = eitptr;
			Start_eid = eitptr->event_id;
			Start_sid = eitptr->servid;
		}
		top->next = eitptr ;
		eitptr->prev = top ;
		return ;
	}
	cur = top->next ;
	if( eitptr->event_status ){
		while( cur->next != NULL )
			cur = cur->next;
		cur->next    = eitptr;
		eitptr->prev = cur;
		return;
	}
	while(cur != NULL){
		rc = memcmp(&cur->yy, &eitptr->yy, (sizeof(int) * 3));
		if(rc == 0){
			rc = memcmp(&cur->hh, &eitptr->hh, (sizeof(int) * 3));
			if(rc == 0){
				cur->prev->next = eitptr;
				eitptr->prev    = cur->prev;
				if( cur->next != NULL ){
					//重複イベント削除
					while(1){
						duration += cur->duration;
						tmp       = cur->next;
						free( cur->title );
						free( cur->subtitle );
						free( cur );
						cur = tmp;
						if( cur != NULL ){
							if( duration >= eitptr->duration ){
								eitptr->next = cur;
								cur->prev    = eitptr;
								return;
							}
						}else{
							eitptr->next = NULL;
							return;
						}
					}
				}
				eitptr->next = NULL;
				free( cur->title );
				free( cur->subtitle );
				free( cur );
				return;
			}
		}
		if(rc > 0){
			cur->prev->next = eitptr;
			eitptr->prev    = cur->prev;
			end_tm = timeParse( eitptr ) + eitptr->duration;
			if( end_tm <= timeParse( cur ) ){
				cur->prev    = eitptr;
				eitptr->next = cur;
				return;
			}else{
				while(1){
					tmp = cur->next;
					free( cur->title );
					free( cur->subtitle );
					free( cur );
					cur = tmp;
					if( cur != NULL ){
						if( end_tm <= timeParse( cur ) ){
							cur->prev    = eitptr;
							eitptr->next = cur;
							return;
						}
					}else{
						eitptr->next = NULL;
						return;
					}
				}
			}
		}
		if( cur->next == NULL ){
			cur->next    = eitptr;
			eitptr->prev = cur;
			return;
		}
		cur = cur->next;
	}
	return ;

}

void dumpEIT(unsigned char *ptr, int serv_id, int original_network_id, int transport_stream_id, EIT_CONTROL *eitsch, EIT_CONTROL *eit_pf, int mode )
{

	EIThead  eith;
	EITbody  eitb;
	SEVTdesc sevtd;

	EEVTDhead eevthead;
	EEVTDitem eevtitem;
	EEVTDtail eevttail;

	EEVTDitem save_eevtitem;

	EIT_CONTROL	*cur;
	EIT_CONTROL	*eittop;

	int len = 0;
	int loop_len = 0;
	int loop_blen = 0;
	int loop_elen = 0;

	/* EIT */
	len = parseEIThead(ptr, &eith);

	if( mode && eith.table_id!=0x4EU )
		return;

	eittop = eith.table_id!=0x4EU ? eitsch : eit_pf;
	ptr += len;
	loop_len = eith.section_length - (len - 3 + 4); // 3は共通ヘッダ長 4はCRC
	while(loop_len > 0) {
		/* 連続する拡張イベントは、漢字コードが泣き別れして
		   分割されるようだ。連続かどうかは、item_description_lengthが
		   設定されているかどうかで判断できるようだ。 */
		memset(&save_eevtitem, 0, sizeof(EEVTDitem));

		len = parseEITbody(ptr, &eitb);
		ptr += len;
		loop_len -= len;
    
		/* printf("evtid:%d\n", eitb.event_id); */
    
		loop_blen = eitb.descriptors_loop_length;
		loop_len -= loop_blen;
		while(loop_blen > 0) {

			len = parseSEVTdesc(ptr, &sevtd);
			if(len > 0) {

				/*
				  if(eith.service_id == 19304 && 
				  eitb.event_id == 46564) {
				  printf("aa");
				  }
				*/

				if(eith.service_id != serv_id){
					ptr += len;
					loop_blen -= len;
					continue ;
				}

				if( eitb.event_status != EVENT_UNCERTAINTY ){
					cur = searcheit(eittop, eith.service_id, eitb.event_id);
					if(cur == NULL){
						cur = calloc(1, sizeof(EIT_CONTROL));
						cur->table_id       = eith.table_id ;
						cur->event_id       = eitb.event_id ;
						cur->servid         = eith.service_id ;
						cur->running_status = eitb.running_status;
						cur->yy             = eitb.yy;
						cur->mm             = eitb.mm;
						cur->dd             = eitb.dd;
						cur->hh             = eitb.hh;
						cur->hm             = eitb.hm;
						cur->ss             = eitb.ss;
						cur->duration       = eitb.duration;
						cur->event_status   = eitb.event_status;
						cur->title = calloc(1, (strlen(sevtd.event_name) + 1));
						memcpy(cur->title, sevtd.event_name, strlen(sevtd.event_name));
						cur->subtitle = calloc(1, (strlen(sevtd.text) + 1));
						memcpy(cur->subtitle, sevtd.text, strlen(sevtd.text));
						enqueue(eittop, cur);
					}else{
						cur->running_status = eitb.running_status;
						cur->yy             = eitb.yy;
						cur->mm             = eitb.mm;
						cur->dd             = eitb.dd;
						cur->hh             = eitb.hh;
						cur->hm             = eitb.hm;
						cur->ss             = eitb.ss;
						cur->duration       = eitb.duration;
						cur->event_status   = eitb.event_status;
						if( eith.table_id==0x4EU && eitb.event_status ){
							//上書での時系列連続性の判断が出来なくなるため以降を削除
							EIT_CONTROL	*cur_tmp;

							cur_tmp   = cur->next;
							cur->next = NULL;
							while( cur_tmp != NULL ){
								cur = cur_tmp->next;
								free( cur_tmp->title );
								free( cur_tmp->subtitle );
								free( cur_tmp );
								cur_tmp = cur;
							}
						}
					}
				}
			} else {
				len = parseEEVTDhead(ptr, &eevthead);

				/*
				  if(eith.service_id == 19304 && 
				  eitb.event_id == 46564) {
				  printf("aa");
				  }
				*/


				if(len > 0) {
					ptr += len;
					loop_blen -= len;

					loop_elen = eevthead.length_of_items;
					loop_len -= loop_elen;
					while(loop_elen > 0) {
						len = parseEEVTDitem(ptr, &eevtitem);

						ptr += len;
						loop_elen -= len;
						loop_blen -= len;

						if(checkEEVTDitem(&save_eevtitem, &eevtitem, 
										  eevthead.descriptor_number)) {
#if 0
							if(mode == 1) { /* long format */
								fprintf(out, "EEVT,%d,%d,%d,%s,%s\n", 
										eith.service_id,
										eitb.event_id,
										eevtitem.descriptor_number, /* 退避項目 */
										eevtitem.item_description,
										eevtitem.item);
							}
#endif
						}
					}

					len = parseEEVTDtail(ptr, &eevttail);
#if 0
					if(mode == 1) { /* long format */
						fprintf(out, "EEVTt,%d,%d,%d,%s\n", 
								eith.service_id,
								eitb.event_id,
								eevthead.descriptor_number,
								eevttail.text);
					}
#endif
				} else {
					ContentDesc contentDesc;
					len = parseContentDesc(ptr, &contentDesc);
					if (len > 0) {
//						int header_printed = 0;
						for (int i = 0; i < contentDesc.descriptor_length - 1; i+=2) {
							/*
							if (0xff == (unsigned char)contentDesc.content[i])
								continue;
							*/
#if 0
							if (!header_printed) {
								fprintf(out, "Content,%d,%d",
									eith.service_id,
									eitb.event_id);
								header_printed = 1;
							}
#endif
#if 0
							fprintf(out, ",%02x%02x", (unsigned char)contentDesc.content[i], (unsigned char)contentDesc.content[i+1]);
#endif
						}
						if((eith.original_network_id == original_network_id) && (eith.transport_stream_id == transport_stream_id)){
							cur = searcheit(eittop, eith.service_id, eitb.event_id);
							if(cur != NULL){
								unsigned char	*genre_p = (unsigned char *)contentDesc.content;

								cur->content_type    = *genre_p >> 4;
								cur->content_subtype = *genre_p & 0x0fu;
								if( contentDesc.descriptor_length >= 4 ){
									cur->genre2     = genre_p[2] >> 4;
									cur->sub_genre2 = genre_p[2]!=0xE0u ? genre_p[2] & 0x0fu : genre_p[3];
									if( contentDesc.descriptor_length >= 6 ){
										cur->genre3     = genre_p[4] >> 4;
										cur->sub_genre3 = genre_p[4]!=0xE0u ? genre_p[4] & 0x0fu : genre_p[5];
									}else{
										cur->genre3     = 16;
										cur->sub_genre3 = 16;
									}
								}else{
									cur->genre2     = 16;
									cur->sub_genre2 = 16;
									cur->genre3     = 16;
									cur->sub_genre3 = 16;
								}
#if 0
								fprintf(stdout, "%s:", cur->title);
								fprintf(stdout, ",%02x%02x", (unsigned char)contentDesc.content[0], (unsigned char)contentDesc.content[1]);
								fprintf(stdout, ",%02x%02x\n", (unsigned char)contentDesc.content[2], (unsigned char)contentDesc.content[3]);
#endif

							}
#if 0
							if (header_printed) {
								fprintf(out, "\n");
							}
#endif
						}
					} else {
						SeriesDesc seriesDesc;
						len = parseSeriesDesc(ptr, &seriesDesc);
						if (len > 0) {
							cur = searcheit(eittop, eith.service_id, eitb.event_id);
							if(cur != NULL)
								cur->episode_number = seriesDesc.episode_number;
#if 0
							fprintf(out, "Series,%d,%d,series=%d,repeat=%01x,pattern=%d,expire_valid=%d,expire=%04x,epinum=%d,lastepinum=%d,%s\n",
								eith.service_id,
								eitb.event_id,
								seriesDesc.series_id,
								seriesDesc.repeat_label,
								seriesDesc.program_pattern,
								seriesDesc.expire_date_valid_flag,
								seriesDesc.expire_date,
								seriesDesc.episode_number,
								seriesDesc.last_episode_number,
								seriesDesc.series_name_char);
#endif
						} else {
							len = parseOTHERdesc(ptr);
						}
					}
				}
			}
			ptr += len;
			loop_blen -= len;
		}
#if 0
		/* 最後のブレークチェック */
		if(checkEEVTDitem(&save_eevtitem, NULL, 0)) {
			if(mode == 1) { /* long format */
				fprintf(out, "EEVT,%d,%d,%d,%s,%s\n", 
						eith.service_id,
						eitb.event_id,
						save_eevtitem.descriptor_number,
						save_eevtitem.item_description,
						save_eevtitem.item);
			}
		}
#endif
	}

	return;
}
