<?php
include_once( INSTALL_PATH . '/Keyword.class.php' );
include_once( INSTALL_PATH . "/reclib.php" );

define( 'CERTAINTY', 0 );
define( 'START_TIME_UNCERTAINTY', 1 );
define( 'DURATION_UNCERTAINTY', 2 );
define( 'NEXT_EVENT_UNCERTAINTY', 4 );

function garbageClean() {
	// 不要なプログラムの削除
	// 8日以上前のプログラムを消す
	$arr = array();
	$arr = DBRecord::createRecords(  PROGRAM_TBL, "WHERE endtime < subdate( now(), 2 )" );
	foreach( $arr as $val ) $val->delete();
	
	// 8日以上先のデータがあれば消す
	$arr = array();
	$arr = DBRecord::createRecords(  PROGRAM_TBL, "WHERE starttime  > adddate( now(), 8 ) ");
	foreach( $arr as $val ) $val->delete();

	// 8日以上前のログを消す
	$arr = array();
	$arr = DBRecord::createRecords(  LOG_TBL, "WHERE logtime < subdate( now(), 8 )" );
	foreach( $arr as $val ) $val->delete();
}

function doKeywordReservation( $ch_id = 0 ) {
  // キーワード自動録画予約
 	$arr = array();
	$arr = Keyword::createKeywords();
	foreach( $arr as $val ) {
		try {
			$val->reservation($ch_id);
		}
		catch( Exception $e ) {
			// 無視
		}
	}
}

function storeProgram( $type, $xmlfile ) {
	global $BS_CHANNEL_MAP, $GR_CHANNEL_MAP, $CS_CHANNEL_MAP;
	global $settings;
	
	// チャンネルマップファイルの準備
	$map = array();
	if( $type == "BS" ) $map = $BS_CHANNEL_MAP;
	else if( $type == "GR") $map = $GR_CHANNEL_MAP;
	else if( $type == "CS") $map = $CS_CHANNEL_MAP;
	
	// XML parse
  	$xml = @simplexml_load_file( $xmlfile );
	if( $xml === false ) {
		reclog( "getepg:: 正常な".$xmlfile."が作成されなかった模様(放送間帯でないなら問題ありません)", EPGREC_WARN );
		$new_name = $xmlfile.'.'.toDatetime(time());
		rename( $xmlfile, $new_name );
		return -1;	// XMLが読み取れないなら何もしない
	}
	// channel抽出
	foreach( $xml->channel as $ch ) {
		$disc = $ch['id'];
	 try {
		// チャンネルデータを探す
		$num = DBRecord::countRecords( CHANNEL_TBL , "WHERE channel_disc = '" . $disc ."'" );
		if( $num == 0 ) {
			if( array_key_exists( "$disc", $map ) ) {
				// チャンネルデータがないなら新規作成
				$rec = new DBRecord( CHANNEL_TBL );
				$rec->type = $type;
				$rec->channel = $map["$disc"];
				$rec->channel_disc = $disc;
				$rec->name = $ch->{'display-name'};
			}
		}
		else {
			$rec = new DBRecord(CHANNEL_TBL, "channel_disc", $disc );
			if( $rec->name != $ch->{'display-name'} ){
				reclog( 'getepg::チャンネル名が違います('.$xmlfile.')', EPGREC_WARN );
				$new_name = $xmlfile.'.'.toDatetime(time());
				rename( $xmlfile, $new_name );
				return -1;	//信頼できないデータなので終了
			}
		}
	 }
	 catch( Exception $e ) {
		reclog( "getepg::DBの接続またはチャンネルテーブルの書き込みに失敗", EPGREC_ERROR );
	 }
	}
	// channel 終了

	// EIT[pf]数取得
	$pf_lmt = count( $xml->programme_pf );
	if( $pf_lmt == 0 ){
		reclog( 'getepg:: 正常な'.$xmlfile.'が作成されなかった模様(EITが無い)', EPGREC_WARN );
		$new_name = $xmlfile.'.'.toDatetime(time());
		rename( $xmlfile, $new_name );
		return -1;	// XMLが読み取れないなら何もしない
	}

	for( $cnt=0; $cnt<$pf_lmt; $cnt++ ) {
			$event_pf[$cnt]->starttime    = $xml->programme_pf[$cnt]['start'];
			$event_pf[$cnt]->endtime      = $xml->programme_pf[$cnt]['stop'];
			$event_pf[$cnt]->channel_disc = $xml->programme_pf[$cnt]['channel'];
			$event_pf[$cnt]->title        = $xml->programme_pf[$cnt]->title;
			$event_pf[$cnt]->desc         = $xml->programme_pf[$cnt]->desc;
			$event_pf[$cnt]->eid          = $xml->programme_pf[$cnt]->eid;
			$event_pf[$cnt]->sub_genre    = $xml->programme_pf[$cnt]->sub_genre;
			$event_pf[$cnt]->sub_genre2   = $xml->programme_pf[$cnt]->sub_genre2;
			$event_pf[$cnt]->sub_genre3   = $xml->programme_pf[$cnt]->sub_genre3;
			$event_pf[$cnt]->status       = $xml->programme_pf[$cnt]->status;

			$cat_ja = "";
			$cat_en = "";
			foreach( $xml->programme_pf[$cnt]->category as $cat ) {
				if( $cat['lang'] == "ja_JP" ) $cat_ja = $cat;
				if( $cat['lang'] == "en" ) $cat_en = $cat;
			}
			$cat_rec = null;
			try {
				// カテゴリを処理する
				$category_disc = md5( $cat_ja . $cat_en );
				$num = DBRecord::countRecords(CATEGORY_TBL, "WHERE category_disc = '".$category_disc."'" );
				if( $num == 0 ) {
					// 新規カテゴリの追加
					$cat_rec = new DBRecord( CATEGORY_TBL );
					$cat_rec->name_jp = $cat_ja;
					$cat_rec->name_en = $cat_en;
					$cat_rec->category_disc = $category_disc;
					reclog("getepg:: 新規カテゴリ".$cat_ja."を追加" );
				}
				else
					$cat_rec = new DBRecord(CATEGORY_TBL, "category_disc" , $category_disc );
				$event_pf[$cnt]->category = $cat_rec->id;
			}
			catch( Exception $e ) {
				reclog("getepg:: カテゴリテーブルのアクセスに失敗した模様", EPGREC_ERROR );
				reclog("getepg:: ".$e->getMessage()."" ,EPGREC_ERROR );
				exit( $e->getMessage() );
			}
			if( $xml->programme_pf[$cnt]->genre2 != 16 ){
				$work = $xml->programme_pf[$cnt]->genre2;
				$genre_rec = new DBRecord(CATEGORY_TBL, "name_jp" , $work );
				$event_pf[$cnt]->genre2 = $genre_rec->id;
			}else
				$event_pf[$cnt]->genre2 = 0;
			if( $xml->programme_pf[$cnt]->genre3 != 16 ){
				$work = $xml->programme_pf[$cnt]->genre3;
				$genre_rec = new DBRecord(CATEGORY_TBL, "name_jp" , $work );
				$event_pf[$cnt]->genre3 = $genre_rec->id;
			}else
				$event_pf[$cnt]->genre3 = 0;
	}


	// EIT[schedule]数取得 ない場合はEIT[pf]以降3h分をDBから補完
	$ev_lmt = count( $xml->programme );
	if( $ev_lmt == 0 ){
		$pf_last  = $pf_lmt - 1;
		$after_3H = time() + 3*60*60;
		if( $event_pf[$pf_last]->status == CERTAINTY ){
			$pf_endtm = toTimestamp( $event_pf[$pf_last]->endtime );
			$epg_end  = $after_3H<$pf_endtm ? $pf_endtm+2*60*60 : $after_3H;
		}else
			if( $event_pf[$pf_last]->status == DURATION_UNCERTAINTY ){
				$pf_endtm = toTimestamp( $event_pf[$pf_last]->starttime );
				$epg_end  = $after_3H<$pf_endtm+1*60*60 ? $pf_endtm+3*60*60 : $after_3H;
			}else
				$epg_end  = $after_3H + 2*60*60;
		$options = "WHERE channel_disc = '".$event_pf[0]->channel_disc."' ".
			"AND starttime < '".toDatetime($epg_end)."' AND endtime > '".$event_pf[0]->starttime."' ORDER BY starttime ASC";
		$ev_lmt = DBRecord::countRecords( PROGRAM_TBL, $options );
		if( $ev_lmt > 0 )
			$records = DBRecord::createRecords( PROGRAM_TBL, $options );
		for( $cnt=0; $cnt<$ev_lmt; $cnt++ ){
			$event_sch[$cnt]->starttime    = $records[$cnt]->starttime;
			$event_sch[$cnt]->endtime      = $records[$cnt]->endtime;
			$event_sch[$cnt]->channel_disc = $records[$cnt]->channel_disc;
			$event_sch[$cnt]->title        = $records[$cnt]->title;
			$event_sch[$cnt]->desc         = $records[$cnt]->description;
			$event_sch[$cnt]->eid          = $records[$cnt]->eid;
			$event_sch[$cnt]->category     = $records[$cnt]->category_id;
			$event_sch[$cnt]->genre2       = $records[$cnt]->genre2;
			$event_sch[$cnt]->genre3       = $records[$cnt]->genre3;
			$event_sch[$cnt]->sub_genre    = $records[$cnt]->sub_genre;
			$event_sch[$cnt]->sub_genre2   = $records[$cnt]->sub_genre2;
			$event_sch[$cnt]->sub_genre3   = $records[$cnt]->sub_genre3;
			$event_sch[$cnt]->status       = CERTAINTY;
		}
	}else{
		for( $cnt=0; $cnt<$ev_lmt; $cnt++ ){
			$event_sch[$cnt]->starttime    = $xml->programme[$cnt]['start'];
			$event_sch[$cnt]->endtime      = $xml->programme[$cnt]['stop'];
			$event_sch[$cnt]->channel_disc = $xml->programme[$cnt]['channel'];
			$event_sch[$cnt]->title        = $xml->programme[$cnt]->title;
			$event_sch[$cnt]->desc         = $xml->programme[$cnt]->desc;
			$event_sch[$cnt]->eid          = $xml->programme[$cnt]->eid;
			$event_sch[$cnt]->sub_genre    = $xml->programme[$cnt]->sub_genre;
			$event_sch[$cnt]->sub_genre2   = $xml->programme[$cnt]->sub_genre2;
			$event_sch[$cnt]->sub_genre3   = $xml->programme[$cnt]->sub_genre3;
			$event_sch[$cnt]->status       = CERTAINTY;

			$cat_ja = "";
			$cat_en = "";
			foreach( $xml->programme[$cnt]->category as $cat ) {
				if( $cat['lang'] == "ja_JP" ) $cat_ja = $cat;
				if( $cat['lang'] == "en" ) $cat_en = $cat;
			}
			$cat_rec = null;
			try {
				// カテゴリを処理する
				$category_disc = md5( $cat_ja . $cat_en );
				$num = DBRecord::countRecords(CATEGORY_TBL, "WHERE category_disc = '".$category_disc."'" );
				if( $num == 0 ) {
					// 新規カテゴリの追加
					$cat_rec = new DBRecord( CATEGORY_TBL );
					$cat_rec->name_jp = $cat_ja;
					$cat_rec->name_en = $cat_en;
					$cat_rec->category_disc = $category_disc;
					reclog("getepg:: 新規カテゴリ".$cat_ja."を追加" );
				}
				else
					$cat_rec = new DBRecord(CATEGORY_TBL, "category_disc" , $category_disc );
				$event_sch[$cnt]->category = $cat_rec->id;
			}
			catch( Exception $e ) {
				reclog("getepg:: カテゴリテーブルのアクセスに失敗した模様", EPGREC_ERROR );
				reclog("getepg:: ".$e->getMessage()."" ,EPGREC_ERROR );
				exit( $e->getMessage() );
			}
			if( $xml->programme[$cnt]->genre2 != 16 ){
				$work = $xml->programme[$cnt]->genre2;
				$genre_rec = new DBRecord(CATEGORY_TBL, "name_jp" , $work );
				$event_sch[$cnt]->genre2 = $genre_rec->id;
			}else
				$event_sch[$cnt]->genre2 = 0;
			if( $xml->programme[$cnt]->genre3 != 16 ){
				$work = $xml->programme[$cnt]->genre3;
				$genre_rec = new DBRecord(CATEGORY_TBL, "name_jp" , $work );
				$event_sch[$cnt]->genre3 = $genre_rec->id;
			}else
				$event_sch[$cnt]->genre3 = 0;
		}
	}

	// EIT[schedule]放送休止挿入 fsusb2nが停波中に受信を行うとスタックするためその回避に使用する
	for( $ev_cnt=1; $ev_cnt<$ev_lmt; $ev_cnt++ ){
		if( strcmp( $event_sch[$ev_cnt-1]->channel_disc, $event_sch[$ev_cnt]->channel_disc ) == 0 ){
			if( strcmp( $event_sch[$ev_cnt-1]->endtime, $event_sch[$ev_cnt]->starttime ) != 0 ){
//				array_splice( $event_sch, $ev_cnt, 0, $event_sch[$ev_cnt] );		//番組挿入
				for( $ev_sft=$ev_lmt; $ev_sft>$ev_cnt; $ev_sft-- ){
//					$event_sch[$ev_sft] = $event_sch[$ev_sft-1];
					$event_sch[$ev_sft]->starttime    = $event_sch[$ev_sft-1]->starttime;
					$event_sch[$ev_sft]->endtime      = $event_sch[$ev_sft-1]->endtime ;
					$event_sch[$ev_sft]->channel_disc = $event_sch[$ev_sft-1]->channel_disc;
					$event_sch[$ev_sft]->title        = $event_sch[$ev_sft-1]->title;
					$event_sch[$ev_sft]->desc         = $event_sch[$ev_sft-1]->desc;
					$event_sch[$ev_sft]->eid          = $event_sch[$ev_sft-1]->eid;
					$event_sch[$ev_sft]->category     = $event_sch[$ev_sft-1]->category;
					$event_sch[$ev_sft]->genre2       = $event_sch[$ev_sft-1]->genre2;
					$event_sch[$ev_sft]->genre3       = $event_sch[$ev_sft-1]->genre3;
					$event_sch[$ev_sft]->sub_genre    = $event_sch[$ev_sft-1]->sub_genre;
					$event_sch[$ev_sft]->sub_genre2   = $event_sch[$ev_sft-1]->sub_genre2;
					$event_sch[$ev_sft]->sub_genre3   = $event_sch[$ev_sft-1]->sub_genre3;
					$event_sch[$ev_sft]->status       = $event_sch[$ev_sft-1]->status;
				}
				$event_sch[$ev_cnt]->starttime    = $event_sch[$ev_cnt-1]->endtime;
				$event_sch[$ev_cnt]->endtime      = $event_sch[$ev_cnt+1]->starttime;
//				$event_sch[$ev_cnt]->channel_disc = $event_sch[$ev_cnt-1]->channel_disc;
				$event_sch[$ev_cnt]->title        = '放送休止';
				$event_sch[$ev_cnt]->desc         = "";
				$event_sch[$ev_cnt]->eid          = -1;
				$event_sch[$ev_cnt]->category     = 1;
				$event_sch[$ev_cnt]->genre2       = 0;
				$event_sch[$ev_cnt]->genre3       = 0;
				$event_sch[$ev_cnt]->sub_genre    = 16;
				$event_sch[$ev_cnt]->sub_genre2   = 16;
				$event_sch[$ev_cnt]->sub_genre3   = 16;
				$event_sch[$ev_cnt]->status       = CERTAINTY;
//reclog( $event_sch[$ev_cnt-1]->starttime.' '.$event_sch[$ev_cnt-1]->endtime.' '.$event_sch[$ev_cnt-1]->title );
//reclog( $event_sch[$ev_cnt]->starttime.' '.$event_sch[$ev_cnt]->endtime.' '.$event_sch[$ev_cnt]->title );
//reclog( $event_sch[$ev_cnt+1]->starttime.' '.$event_sch[$ev_cnt+1]->endtime.' '.$event_sch[$ev_cnt+1]->title );
				$ev_lmt++;
				$ev_cnt++;
			}
		}
	}

	// EIT[schedule]上のEIT[pf]位置取得
	for( $cnt=0; $cnt<$pf_lmt; $cnt++ ){
		$pf_start[$cnt] = -1;
		for( $ev_cnt=0; $ev_cnt<$ev_lmt; $ev_cnt++ ){
			if( strcmp( $event_sch[$ev_cnt]->channel_disc, $event_pf[$cnt]->channel_disc ) == 0 )
				if( strcmp( $event_sch[$ev_cnt]->eid, $event_pf[$cnt]->eid ) == 0 ){
					$pf_start[$cnt] = $ev_cnt;
					break;
				}
		}
	}

	// EIT[schedule]とEIT[pf]をマージ
	for( $ev_cnt=0; $ev_cnt<$ev_lmt; $ev_cnt++ ){
		if( strcmp( $event_sch[$ev_cnt]->channel_disc, $event_pf[0]->channel_disc ) == 0 ){
			$cmp = strcmp( $event_sch[$ev_cnt]->starttime, $event_pf[0]->starttime );
			if( $cmp >= 0 ){
$debug_msg = $event_pf[0]->channel_disc.' (';
				if( $pf_start[0] != -1 ){
					if( $pf_start[0] > $ev_cnt ){
						$erase_cnt = $pf_start[0] - $ev_cnt;
						array_splice( $event_sch, $ev_cnt, $erase_cnt );
if( $ev_lmt == count( $event_sch ) )
$debug_msg .= '[array_splice() not delete!!]';
else
$debug_msg .= '[array_splice():'.$ev_lmt.'>'.count( $event_sch ).']';
						$ev_lmt = count( $event_sch );
						for( $e_cnt=1; $e_cnt<$pf_lmt; $e_cnt++ )
							if( $pf_start[$e_cnt] != -1 )
								$pf_start[$e_cnt] -= $erase_cnt;
					}else
						$ev_cnt = $pf_start[0];
					$event_sch[$ev_cnt]->title      = $event_pf[0]->title;
					$event_sch[$ev_cnt]->desc       = $event_pf[0]->desc;
					$event_sch[$ev_cnt]->category   = $event_pf[0]->category;
					$event_sch[$ev_cnt]->genre2     = $event_pf[0]->genre2;
					$event_sch[$ev_cnt]->genre3     = $event_pf[0]->genre3;
					$event_sch[$ev_cnt]->sub_genre  = $event_pf[0]->sub_genre;
					$event_sch[$ev_cnt]->sub_genre2 = $event_pf[0]->sub_genre2;
					$event_sch[$ev_cnt]->sub_genre3 = $event_pf[0]->sub_genre3;
					$event_sch[$ev_cnt]->status     = $event_pf[0]->status;
					if( $event_pf[0]->status == DURATION_UNCERTAINTY ){
						if( $pf_lmt==1 || $event_pf[1]->status==START_TIME_UNCERTAINTY ){
							$duration = toTimestamp( $event_sch[$ev_cnt]->endtime ) - toTimestamp( $event_sch[$ev_cnt]->starttime );
							$end_time = toTimestamp( $event_pf[0]->starttime ) + $duration;
							$now_time = time();
							$now_time = $now_time + 60 - $now_time % 60;
							if( $end_time < $now_time )
								$end_time = $now_time;
							$next_start = toDatetime( $end_time+60 );		//1min add
						}else
							$next_start = toDatetime( toTimestamp( $event_pf[1]->starttime )+60 );
					}else
						$next_start = $event_pf[0]->endtime;
					if( $cmp != 0 )
						$event_sch[$ev_cnt]->starttime = $event_pf[0]->starttime;
				}else{
$debug_msg .= '[pf incert]';
					// 番組挿入
//					array_splice( $event_sch, $ev_cnt, 0, $event_pf[0] );		//番組挿入
					for( $ev_sft=$ev_lmt; $ev_sft>$ev_cnt; $ev_sft-- ){
//						$event_sch[$ev_sft] = $event_sch[$ev_sft-1];
						$event_sch[$ev_sft]->starttime    = $event_sch[$ev_sft-1]->starttime;
						$event_sch[$ev_sft]->endtime      = $event_sch[$ev_sft-1]->endtime ;
						$event_sch[$ev_sft]->channel_disc = $event_sch[$ev_sft-1]->channel_disc;
						$event_sch[$ev_sft]->title        = $event_sch[$ev_sft-1]->title;
						$event_sch[$ev_sft]->desc         = $event_sch[$ev_sft-1]->desc;
						$event_sch[$ev_sft]->eid          = $event_sch[$ev_sft-1]->eid;
						$event_sch[$ev_sft]->category     = $event_sch[$ev_sft-1]->category;
						$event_sch[$ev_sft]->genre2       = $event_sch[$ev_sft-1]->genre2;
						$event_sch[$ev_sft]->genre3       = $event_sch[$ev_sft-1]->genre3;
						$event_sch[$ev_sft]->sub_genre    = $event_sch[$ev_sft-1]->sub_genre;
						$event_sch[$ev_sft]->sub_genre2   = $event_sch[$ev_sft-1]->sub_genre2;
						$event_sch[$ev_sft]->sub_genre3   = $event_sch[$ev_sft-1]->sub_genre3;
						$event_sch[$ev_sft]->status       = $event_sch[$ev_sft-1]->status;
					}
//					$event_sch[$ev_cnt] = $event_pf[0];		//番組挿入
					$event_sch[$ev_cnt]->starttime    = $event_pf[0]->starttime;
					$event_sch[$ev_cnt]->endtime      = $event_pf[0]->endtime ;
					$event_sch[$ev_cnt]->channel_disc = $event_pf[0]->channel_disc;
					$event_sch[$ev_cnt]->title        = $event_pf[0]->title;
					$event_sch[$ev_cnt]->desc         = $event_pf[0]->desc;
					$event_sch[$ev_cnt]->eid          = $event_pf[0]->eid;
					$event_sch[$ev_cnt]->category     = $event_pf[0]->category;
					$event_sch[$ev_cnt]->genre2       = $event_pf[0]->genre2;
					$event_sch[$ev_cnt]->genre3       = $event_pf[0]->genre3;
					$event_sch[$ev_cnt]->sub_genre    = $event_pf[0]->sub_genre;
					$event_sch[$ev_cnt]->sub_genre2   = $event_pf[0]->sub_genre2;
					$event_sch[$ev_cnt]->sub_genre3   = $event_pf[0]->sub_genre3;
					$event_sch[$ev_cnt]->status       = $event_pf[0]->status;
					$ev_lmt++;
					for( $e_cnt=1; $e_cnt<$pf_lmt; $e_cnt++ )
						if( $pf_start[$e_cnt] != -1 )
							$pf_start[$e_cnt]++;
					if( $event_pf[0]->status == DURATION_UNCERTAINTY ){
						if( $pf_lmt==1 || $event_pf[1]->status==START_TIME_UNCERTAINTY ){
							$end_time = toTimestamp( $event_sch[$ev_cnt+1]->starttime );
							$now_time = time();
							$now_time = $now_time + 60 - $now_time % 60;
							if( $end_time < $now_time )
								$end_time = $now_time;
							$next_start = toDatetime( $end_time+60 );		//1min add
						}else
							$next_start = toDatetime( toTimestamp( $event_pf[1]->starttime )+60 );
					}else
						$next_start = $event_pf[0]->endtime;
				}
				$event_sch[$ev_cnt]->endtime = $next_start;
				$cnt = 1;
				$ev_cnt++;
$debug_msg .= ' '.$cnt.':'.$next_start.')';

				while( $cnt < $pf_lmt ){
$debug_msg .= ' (';
					if( $pf_start[$cnt] != -1 ){
						if( $pf_start[$cnt] > $ev_cnt ){
							$erase_cnt = $pf_start[$cnt] - $ev_cnt;
							array_splice( $event_sch, $ev_cnt, $erase_cnt );
if( $ev_lmt == count( $event_sch ) )
$debug_msg .= '[array_splice() not delete!!]';
else
$debug_msg .= '[array_splice():'.$ev_lmt.'>'.count( $event_sch ).']:';
							$ev_lmt = count( $event_sch );
							for( $e_cnt=$cnt+1; $e_cnt<$pf_lmt; $e_cnt++ )
								if( $pf_start[$e_cnt] != -1 )
									$pf_start[$e_cnt] -= $erase_cnt;
						}else
							$ev_cnt = $pf_start[$cnt];
						$event_sch[$ev_cnt]->channel_disc = $event_pf[$cnt]->channel_disc;
						$event_sch[$ev_cnt]->title        = $event_pf[$cnt]->title;
						$event_sch[$ev_cnt]->desc         = $event_pf[$cnt]->desc;
						$event_sch[$ev_cnt]->category     = $event_pf[$cnt]->category;
						$event_sch[$ev_cnt]->genre2       = $event_pf[$cnt]->genre2;
						$event_sch[$ev_cnt]->genre3       = $event_pf[$cnt]->genre3;
						$event_sch[$ev_cnt]->sub_genre    = $event_pf[$cnt]->sub_genre;
						$event_sch[$ev_cnt]->sub_genre2   = $event_pf[$cnt]->sub_genre2;
						$event_sch[$ev_cnt]->sub_genre3   = $event_pf[$cnt]->sub_genre3;
						$event_sch[$ev_cnt]->status       = $event_pf[$cnt]->status;
						switch( $event_pf[$cnt]->status ){
							case START_TIME_UNCERTAINTY:
								$duration = toTimestamp( $event_pf[$cnt]->endtime );
								break;
							case DURATION_UNCERTAINTY:
								if( $cnt<$pf_lmt-1 && $event_pf[$cnt+1]->status!=START_TIME_UNCERTAINTY )
									$duration = toTimestamp($event_pf[$cnt+1]->starttime) - toTimestamp($event_pf[$cnt]->starttime);
								else
									$duration = toTimestamp($event_sch[$ev_cnt]->endtime) - toTimestamp($event_sch[$ev_cnt]->starttime);
								break;
							default:
								$duration = toTimestamp($event_pf[$cnt]->endtime) - toTimestamp($event_pf[$cnt]->starttime);
								break;
						}
					}else{
$debug_msg .= '[pf incert]';
//						array_splice( $event_sch, $ev_cnt, 0, $event_pf[$cnt] );		//番組挿入
						for( $ev_sft=$ev_lmt; $ev_sft>$ev_cnt; $ev_sft-- ){
//							$event_sch[$ev_sft] = $event_sch[$ev_sft-1];
							$event_sch[$ev_sft]->starttime    = $event_sch[$ev_sft-1]->starttime;
							$event_sch[$ev_sft]->endtime      = $event_sch[$ev_sft-1]->endtime ;
							$event_sch[$ev_sft]->channel_disc = $event_sch[$ev_sft-1]->channel_disc;
							$event_sch[$ev_sft]->title        = $event_sch[$ev_sft-1]->title;
							$event_sch[$ev_sft]->desc         = $event_sch[$ev_sft-1]->desc;
							$event_sch[$ev_sft]->eid          = $event_sch[$ev_sft-1]->eid;
							$event_sch[$ev_sft]->category     = $event_sch[$ev_sft-1]->category;
							$event_sch[$ev_sft]->genre2       = $event_sch[$ev_sft-1]->genre2;
							$event_sch[$ev_sft]->genre3       = $event_sch[$ev_sft-1]->genre3;
							$event_sch[$ev_sft]->sub_genre    = $event_sch[$ev_sft-1]->sub_genre;
							$event_sch[$ev_sft]->sub_genre2   = $event_sch[$ev_sft-1]->sub_genre2;
							$event_sch[$ev_sft]->sub_genre3   = $event_sch[$ev_sft-1]->sub_genre3;
							$event_sch[$ev_sft]->status       = $event_sch[$ev_sft-1]->status;
						}
//						$event_sch[$ev_cnt] = $event_pf[$cnt];		//番組挿入
						$event_sch[$ev_cnt]->starttime    = $event_pf[$cnt]->starttime;
						$event_sch[$ev_cnt]->endtime      = $event_pf[$cnt]->endtime ;
						$event_sch[$ev_cnt]->channel_disc = $event_pf[$cnt]->channel_disc;
						$event_sch[$ev_cnt]->title        = $event_pf[$cnt]->title;
						$event_sch[$ev_cnt]->desc         = $event_pf[$cnt]->desc;
						$event_sch[$ev_cnt]->eid          = $event_pf[$cnt]->eid;
						$event_sch[$ev_cnt]->category     = $event_pf[$cnt]->category;
						$event_sch[$ev_cnt]->genre2       = $event_pf[$cnt]->genre2;
						$event_sch[$ev_cnt]->genre3       = $event_pf[$cnt]->genre3;
						$event_sch[$ev_cnt]->sub_genre    = $event_pf[$cnt]->sub_genre;
						$event_sch[$ev_cnt]->sub_genre2   = $event_pf[$cnt]->sub_genre2;
						$event_sch[$ev_cnt]->sub_genre3   = $event_pf[$cnt]->sub_genre3;
						$event_sch[$ev_cnt]->status       = $event_pf[$cnt]->status;
						$ev_lmt++;
						for( $e_cnt=$cnt+1; $e_cnt<$pf_lmt; $e_cnt++ )
							if( $pf_start[$e_cnt] != -1 )
								$pf_start[$e_cnt]++;
						switch( $event_pf[$cnt]->status ){
							case START_TIME_UNCERTAINTY:
								$duration = toTimestamp( $event_pf[$cnt]->endtime );
								break;
							case DURATION_UNCERTAINTY:
								if( $cnt<$pf_lmt-1 && $event_pf[$cnt+1]->status!=START_TIME_UNCERTAINTY )
									$duration = toTimestamp($event_pf[$cnt+1]->starttime) - toTimestamp($event_pf[$cnt]->starttime);
								else
									$duration = toTimestamp($event_sch[$ev_cnt+1]->starttime) - toTimestamp($event_pf[$cnt]->starttime);		//不定
								break;
							default:
								$duration = toTimestamp($event_pf[$cnt]->endtime) - toTimestamp($event_pf[$cnt]->starttime);
								break;
						}
					}
					$event_sch[$ev_cnt]->starttime = $next_start;
					$next_start                    = toDatetime( toTimestamp( $next_start ) + $duration );
					$event_sch[$ev_cnt]->endtime   = $next_start;
					$ev_cnt++;
					$cnt++;
$debug_msg .= ' '.$cnt.':'.$next_start.')';
				}
				//マージ以降をタイムシフト
				$ev_lmt   = count( $event_sch );
				$add_time = toTimestamp($next_start) - toTimestamp($event_sch[$ev_cnt]->starttime);
				if( $add_time > 0 ){
					if( $ev_lmt > 10 )
						$ev_lmt = 10;
					while( $ev_cnt < $ev_lmt ){
						if( strcmp( '放送休止', $event_sch[$ev_cnt]->title ) != 0 ){
							$event_sch[$ev_cnt]->starttime = toDatetime( toTimestamp( $event_sch[$ev_cnt]->starttime )+$add_time );
							$event_sch[$ev_cnt]->endtime   = toDatetime( toTimestamp( $event_sch[$ev_cnt]->endtime )+$add_time );
						}else{
							$cmp = toTimestamp( $event_sch[$ev_cnt]->endtime ) - toTimestamp( $event_sch[$ev_cnt]->starttime );
							$cmp -= $add_time;
							if( $cmp > 0 ){
								$event_sch[$ev_cnt]->starttime = toDatetime( toTimestamp( $event_sch[$ev_cnt]->starttime )+$add_time );
								break;
							}
							else{
								array_splice( $event_sch, $ev_cnt, 1 );
								if( $cmp == 0 ){
									break;
								}else{
									//他で調節が必要
									break;
								}
							}
						}
						//reclog( 'debug: '.$cnt.':'.$next_start );
						$ev_cnt++;
					}
				}
				reclog( $debug_msg );
				break;
			}
		}
	}
//for( $ev_cnt=0; $ev_cnt<$ev_lmt && $ev_cnt<10; $ev_cnt++ )
//reclog( $ev_cnt.':'.$event_sch[$ev_cnt]->starttime.' '.$event_sch[$ev_cnt]->title );
//$cnt=0;
	// programme 取得
	foreach( $event_sch as $program ) {
//reclog( 'event count:'.$cnt.':'.$ev_lmt );
		$channel_disc = $program->channel_disc;
		if( ! array_key_exists( "$channel_disc", $map ) ) continue;
		$channel = $map["$channel_disc"];
		
		try {
			$channel_rec = new DBRecord(CHANNEL_TBL, "channel_disc", "$channel_disc" );
		}
		catch( Exception $e ) {
			$channel_rec = null;
			reclog( "getepg::チャンネルレコード $channel_disc が発見できない", EPGREC_ERROR );
		}
		if( $channel_rec == null ) continue;	// あり得ないことが起きた
		if( $channel_rec->skip == 1 ) continue;	// 受信しないチャンネル
		$starttime    = $program->starttime;
		$endtime      = $program->endtime;
		$title        = $program->title;
		$desc         = $program->desc;
		$eid          = $program->eid;
		$category_id  = $program->category;
		$genre2       = $program->genre2;
		$genre3       = $program->genre3;
		$sub_genre    = $program->sub_genre;
		$sub_genre2   = $program->sub_genre2;
		$sub_genre3   = $program->sub_genre3;
		$program_disc = md5( $channel_disc . $eid . $starttime . $endtime );
//reclog( 'event count:'.$cnt++.':'.$ev_lmt.' '.$title );
		// プログラム登録
		try {
			//
			$num = DBRecord::countRecords(PROGRAM_TBL, "WHERE program_disc = '".$program_disc."'" );
			if( $num == 0 ) {
				// 新規番組
				$rewrite_eid = 0;
				// 重複チェック 同時間帯にある番組
				$options = "WHERE channel_disc = '".$channel_disc."' ".
					"AND starttime < '". $endtime ."' AND endtime > '".$starttime."'";
				$battings = DBRecord::countRecords(PROGRAM_TBL, $options );
				if( $battings > 0 ) {
					// 重複発生＝おそらく放映時間の変更
					$records = DBRecord::createRecords(PROGRAM_TBL, $options);
					foreach( $records as $rec ) {
						// 自動録画予約された番組は放映時間変更と同時にいったん削除する
						try {
							$reserve = new DBRecord(RESERVE_TBL, "program_id", $rec->id );
							if( time() >= (toTimestamp($reserve->starttime) - $settings->former_time) ) {
								if( $rec->eid == $eid ){
									$rec->starttime    = $starttime;
									$rec->endtime      = $endtime;
									$rec->program_disc = $program_disc;
									$rewrite_eid       = $eid;
								}
								$rev_ds = '予約ID:'.$reserve->id.' '.$reserve->channel_disc.' '.$reserve->starttime.'『'.$reserve->title.'』';
/*ここから(PT1 only)*/
								//録画延伸指示(チューナ依存)
								if( time()<toTimestamp($reserve->endtime) && $reserve->tuner<TUNER_UNIT1 ){
									$add_time = (int)( ( toTimestamp($endtime) - toTimestamp($reserve->endtime) ) / 60 ) * 60;
									$ps       = search_reccmd( $reserve->id );
									if( $ps !== FALSE ){
										exec( '/usr/local/bin/recpt1ctl --pid '.$ps->pid.' --extend '.$add_time );		//(PT1用)
										$reserve->endtime = toDatetime( toTimestamp($reserve->endtime) + $add_time );
										reclog( $rev_ds.'を('.($add_time/60).'分)延長', EPGREC_WARN );
									}else
										reclog( $rev_ds.'の延長に失敗('.($add_time/60).'分)', EPGREC_WARN );
								}else
/*ここまで(PT1 only)*/
									// すでに開始されている録画は無視する
									reclog( $rev_ds.'は録画開始後に時間変更が発生した可能性がある', EPGREC_WARN );
							}
							else {
								if( $reserve->autorec == 0 ){
									if( $reserve->program_id != 0 ){
										//手動予約のタイムシフト再予約準備
										$add_time = toTimestamp($starttime) - toTimestamp($rec->starttime);
										$st_tm    = toDatetime( toTimestamp($reserve->starttime) + $add_time );
										$ed_tm    = toDatetime( toTimestamp($reserve->endtime) + $add_time );
										$ch_id    = $reserve->channel_id;
										$tl_nm    = $reserve->title;
										$tl_ds    = $reserve->description;
										$ct_id    = $reserve->category_id;
										$pg_id    = $reserve->program_id;
										$at_md    = $reserve->autorec;
										$rs_md    = $reserve->mode;
										$discon   = $reserve->discontinuity;
										$rs_dt    = $reserve->dirty;
										Reservation::cancel( $reserve->id );
										//手動予約はタイムシフト再予約
										// program_idまわりで不整合が生じる EIDを予約テーブルに反映して後でprogram_idを書き換えるべき
										$rval = Reservation::custom( $st_tm, $ed_tm, $ch_id, $tl_nm, $tl_ds, $ct_id, $pg_id, $at_md, $rs_md, $discon, $rs_dt );
									}
									// program_idが0の場合は、キャンセルしない
								}else{
//									reclog( '予約ID'.$reserve->id.': '.$reserve->channel_disc.' '.$reserve->starttime.'『'.$reserve->title.'』は時間変更の可能性があり予約取り消し' );
									Reservation::cancel( $reserve->id );
								}
							}
						}
						catch( Exception $e ) {
							// 無視
						}
						if( $rec->eid != $rewrite_eid ){
							// 番組削除
							reclog( 'getepg::放送時間重複が発生した番組ID'.$rec->id.": ".$rec->channel_disc.' '.$rec->starttime.'『'.$rec->title.'』を削除' );
							$rec->delete();
						}
					}
				}
				// //
				if( $eid != $rewrite_eid ){
					$rec = new DBRecord( PROGRAM_TBL );
					$rec->channel_disc = $channel_disc;
					$rec->channel_id   = $channel_rec->id;
					$rec->type         = $type;
					$rec->channel      = $channel_rec->channel;
					$rec->title        = $title;
					$rec->description  = $desc;
					$rec->category_id  = $category_id;
					$rec->sub_genre    = $sub_genre;
					$rec->genre2       = $genre2;
					$rec->sub_genre2   = $sub_genre2;
					$rec->genre3       = $genre3;
					$rec->sub_genre3   = $sub_genre3;
					$rec->eid          = $eid;
					$rec->starttime    = $starttime;
					$rec->endtime      = $endtime;
					$rec->program_disc = $program_disc;
				}
			}
			else {
				// 番組内容更新
				$rec = new DBRecord( PROGRAM_TBL, "program_disc", $program_disc );
				//枠移動チェック
				if( strcmp( $rec->title, $title ) != 0 ){
					try {
						$reserve = new DBRecord( RESERVE_TBL, "program_id", $rec->id );
						//自動キーワード予約判定
						if( $reserve->autorec && strcmp( $reserve->title, $rec->title ) == 0 )
							Reservation::cancel( $reserve->id );
						else
						if( $reserve->dirty == 0 && toTimestamp($reserve->starttime)-$settings->former_time > time() ){
							// dirtyが立っておらず現在より後の録画予約であるなら
							$reserve->title = $title;
							$reserve->description = $desc;
							reclog( "getepg:: 予約ID".$reserve->id."のEPG情報が更新された" );
						}
					}
					catch( Exception $e ) {
						// 無視する
					}
					$rec->title = $title;
				}
				if( strcmp( $rec->description , $desc ) != 0 )
					$rec->description = $desc;
				if( $rec->category_id != $category_id )
					$rec->category_id = $category_id;
				if( $rec->sub_genre != $sub_genre )
					$rec->sub_genre = $sub_genre;
				if( $rec->genre2 != $genre2 )
					$rec->genre2 = $genre2;
				if( $rec->sub_genre2 != $sub_genre2 )
					$rec->sub_genre2 = $sub_genre2;
				if( $rec->genre3 != $genre3 )
					$rec->genre3 = $genre3;
				if( $rec->sub_genre3 != $sub_genre3 )
					$rec->sub_genre3 = $sub_genre3;
			}
		}
		catch(Exception $e) {
			reclog( "getepg:: プログラムテーブルに問題が生じた模様", EPGREC_ERROR );
			reclog( "getepg:: ".$e->getMessage()."" , EPGREC_ERROR);
			exit( $e->getMessage() );
		}
	}
	// Programme取得完了
	return ($type=="GR") ? $channel_rec->id : 0;
}
?>
